//
//  CWPoint.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWPoint.h"
#import "CWXMLTranslater.h"

@interface CWPointParser : NSObject <CWXMLTranslaterDelegate> {
@private
  NSURL* url;
  NSMutableArray* points;
}

-(id)initWithSearchString:(NSString*)searchString;

-(NSArray*)parsePoints;

@end



@implementation CWPoint

@synthesize ID = _ID;
@synthesize name = _name;
@synthesize stopPoint = _stopPoint;
@synthesize type = _type;

-(NSString*)typeString;
{
  switch (self.type) {
    case CWPointTypeStopArea:
      return @"STOP_AREA";
    case CWPointTypeAddress:
      return @"ADDRESS";
    case CWPointTypePointOfInterest:
      return @"POI";
    default:
      return @"_unknown_";
  }
}

// NOTE: Journey queries must report ADDRESS and POI as same type, for some stupid reason.
static int CWQueryTypeForPointType(CWPointType type) {
  if (type == 0) {
    return 0;
  } else {
    return 1;
  }
}

-(NSString*)queryString;
{
  return [NSString stringWithFormat:@"%%7C%d%%7C%d", self.ID, CWQueryTypeForPointType(self.type)];
}

+(CWPoint*)pointWithID:(NSUInteger)pointId name:(NSString*)name type:(CWPointType)type;
{
	CWPoint* point = [[CWPoint alloc] init];
  if (point) {
		point->_ID = pointId;
    point->_name = [name retain];
    point->_type = type;
  }
  return [point autorelease];
}

+(NSMutableArray*)defaultKnownPoints;
{
  NSMutableArray* points = [NSMutableArray array];
	[points addObject:[CWPoint pointWithID:80000 name:@"Malmö C" type:CWPointTypeStopArea]];
	[points addObject:[CWPoint pointWithID:80120 name:@"Södervärn Malmö" type:CWPointTypeStopArea]];
	[points addObject:[CWPoint pointWithID:81216 name:@"Lund C" type:CWPointTypeStopArea]];
	[points addObject:[CWPoint pointWithID:83241 name:@"Helsingborg C" type:CWPointTypeStopArea]];
	[points addObject:[CWPoint pointWithID:85131 name:@"Eslövs Station" type:CWPointTypeStopArea]];
  return points;
}

+(NSMutableArray*)defaultJourneySearchBookmarks;
{
  return [NSMutableArray array];
}

+(NSArray*)pointsMatchingString:(NSString*)searchString;
{
  CWPointParser* pointParser = [[CWPointParser alloc] initWithSearchString:searchString];
  NSArray* points = [pointParser parsePoints];
  NSLog(@"Points: %@", [points description]);
	[pointParser release];
  return points;
}

-(id)initWithCoder:(NSCoder *)aDecoder;
{
  self = [self init];
  if (self) {
    _ID = [aDecoder decodeIntegerForKey:@"ID"];
    _name = [[aDecoder decodeObjectForKey:@"name"] retain];
    _stopPoint = [[aDecoder decodeObjectForKey:@"stopPoint"] retain];
    _type = [aDecoder decodeIntegerForKey:@"type"];
  }
  return self;
}

-(void)encodeWithCoder:(NSCoder *)aCoder;
{
  [aCoder encodeInteger:self.ID forKey:@"ID"];
  [aCoder encodeObject:self.name forKey:@"name"];
  [aCoder encodeObject:self.stopPoint forKey:@"stopPoint"];
  [aCoder encodeInteger:self.type forKey:@"type"];
}

-(void)dealloc;
{
  [_name release];
  [super dealloc];
}                          

-(UIImage*)imageForType;
{
  return [UIImage imageNamed:[NSString stringWithFormat:@"point-%d.png", self.type]];	  
}

-(UITableViewCellAccessoryType)accessoryType;
{
  return UITableViewCellAccessoryNone;
  //	return self.type == CWPointTypeStopArea ? UITableViewCellAccessoryDetailDisclosureButton : UITableViewCellAccessoryNone;
}

-(BOOL)isEqual:(id)anObject;
{
  if (self == anObject) {
    return YES;
  } else if ([anObject isKindOfClass:[CWPoint class]]) {
    CWPoint* otherPoint = anObject;
    // It is safe to ignore types, as they use different ID ranges anyway.
    return self.ID == otherPoint.ID;
  }
  return NO;
}

-(NSString*)description;
{
  return [NSString stringWithFormat:@"<CWPoint: %d, '%@', %@>", self.ID, self.name, self.typeString];
}

@end

@implementation CWPointParser

+(NSURL*)queryURLForSearchString:(NSString*)searchString;
{
  NSString* escapedSearchString = [searchString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
  NSString* query = [NSString stringWithFormat:@"querypage.asp?inppointto=zq&inppointfr=%@", escapedSearchString];
  query = [[[[NSBundle mainBundle] infoDictionary] objectForKey:@"ServerURL"] stringByAppendingString:query];
  return [NSURL URLWithString:query];
}

-(id)initWithSearchString:(NSString*)searchString;
{
  self = [super init];
  if (self) {
    url = [[CWPointParser queryURLForSearchString:searchString] retain];
    points = nil;
  }
  return self;
}

-(void)dealloc;
{
  [url release];
  [points release];
  [super dealloc];
}

-(NSArray*)parsePoints;
{
  [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
  NSDictionary* translation = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"CWPointParser" ofType:@"plist"]];
	CWXMLTranslater* translater = [[CWXMLTranslater alloc] initWithTranslationPropertyList:translation delegate:self];
	NSError* error = nil;
  if (![translater translateContentsOfURL:url delegate:translater.delegate error:&error]) {
    NSLog(@"Error: %@", [error description]);
    NSLog(@"URL: %@", [url description]);
    //NSLog(@"XML: %@", [NSString stringWithContentsOfURL:url encoding:NSUTF8StringEncoding error:NULL]);
    // Intentionally not an error, because this happens too often!
    /*
  	UIAlertView* alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"FailedNetwork", nil) message:NSLocalizedString(@"FailedNetworkMessage", nil) delegate:nil cancelButtonTitle:NSLocalizedString(@"OK", nil) otherButtonTitles:nil];
    [alert show];
    [alert release];
     */
    points = nil;
  }
  [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
	return [[points retain] autorelease];
}

-(void)xmlTranslater:(CWXMLTranslater*)translater didTranslateObject:(id)anObject forKey:(NSString*)key;
{
  if (points == nil) {
  	points = [NSMutableArray new];
  }
  [points addObject:anObject];
}

-(id)xmlTranslater:(CWXMLTranslater*)translater willTranslateValueOfClass:(Class)aClass withString:(NSString*)aString forKey:(NSString*)key;
{
	if ([key isEqualToString:@"type"]) {
    CWPointType type;
    if ([aString isEqualToString:@"STOP_AREA"]) {
      type = CWPointTypeStopArea;
    } else if ([aString isEqualToString:@"ADDRESS"]) {
      type = CWPointTypeAddress;
    } else if ([aString isEqualToString:@"POI"]) {
      type = CWPointTypePointOfInterest;
    }
    return [NSNumber numberWithInt:(int)type];
  } else if ([key isEqualToString:@"name"]) {
    return [aString capitalizedString];
  }
  return nil;
}

@end
