//
//  CWStopSelectionController.m
//  ResaISkane
//
//  Copyright 2009-2010 Fredrik Olsson. All rights reserved.
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License. 
//  You may obtain a copy of the License at 
// 
//  http://www.apache.org/licenses/LICENSE-2.0 
//  
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS, 
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//

#import "CWPointSelectionController.h"
#import "CWNetworkChecker.h"
#import "UIColor+CWEditableColor.h"

@interface CWPointSelectionController ()

@property(nonatomic, retain) NSArray* searchResultPoints;

@end


@implementation CWPointSelectionController

@synthesize pointType = _pointType;
@synthesize searchResultPoints = _searchResultPoints;
@synthesize searchOnly = searchOnly;

-(id)initWithPointSelectionType:(CWPointSelectionType)pointType delegate:(id<CWPointSelectionControllerDelegate>)delegate;
{
  self = [self initWithNibName:@"CWPointSelectionController" bundle:nil];
	if (self) {
    _pointType = pointType;
    pointSelectionDelegate = delegate;
    sharedModel = [CWModel sharedModel];
    self.title = NSLocalizedString(pointType == CWPointSelectionTypeFrom ? @"From" : @"To", nil);
  }
  return self;
}

-(void)viewDidLoad;
{
	self.navigationItem.rightBarButtonItem = [self editButtonItem]; 
}

-(void)dealloc;
{
  [_searchResultPoints release];
  [queue release];
	[super dealloc];
}

-(BOOL)tableView:(UITableView*)tableView canSelectPointAtIndex:(NSInteger)index;
{
  CWPoint* point;
	if (self.tableView == tableView) {
  	point = [sharedModel.knownPoints objectAtIndex:index];
  } else {
    point = [self.searchResultPoints objectAtIndex:index];
  }
  return ![point isEqual:self.pointType == CWPointSelectionTypeFrom ? sharedModel.to : sharedModel.from];
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar;
{
	if (![CWNetworkChecker isNetworkAvailable]) {
  	UIAlertView* alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"NoNetwork", nil) message:NSLocalizedString(@"NoNetworkStops", nil) delegate:nil cancelButtonTitle:NSLocalizedString(@"OK", nil) otherButtonTitles:nil];
    [alert show];
    [alert release];
    return NO;
  } else {
  	return YES;
  }
}

- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchString:(NSString *)searchString;
{
	if (queue == nil) {
		queue = [[NSOperationQueue alloc] init];
    [queue setMaxConcurrentOperationCount:1];
  }
  [queue cancelAllOperations];
  NSOperation* op = [[NSInvocationOperation alloc] initWithTarget:self selector:@selector(performSearchWithString:) object:searchString];
  [queue addOperation:op];
  [op release];
  return NO;
}


- (void)searchBarCancelButtonClicked:(UISearchBar *)searchBar;
{
	if (searchOnly) {
		[self.navigationController popToRootViewControllerAnimated:YES];
  }
}

-(void)performSearchWithString:(NSString*)searchString;
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	NSArray* searchResults = [CWPoint pointsMatchingString:searchString];
  self.searchResultPoints = searchResults;
  [self.searchDisplayController.searchResultsTableView reloadData];
  [pool release];
}


#pragma mark Table view methods

// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section;
{
  if (self.tableView == tableView) {
  	return [sharedModel.knownPoints count];
  } else {
    return [self.searchResultPoints count];
  }
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath;
{    
  static NSString *CellIdentifier = @"Cell";
  
  UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
  if (cell == nil) {
    cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
  	cell.textLabel.adjustsFontSizeToFitWidth = YES;
  }

  CWPoint* point = nil;
	if (self.tableView == tableView) {
		point = [sharedModel.knownPoints objectAtIndex:indexPath.row]; 
    cell.accessoryType = [point accessoryType];
  } else {
    point = [self.searchResultPoints objectAtIndex:indexPath.row];
  }
  cell.accessoryType = UITableViewCellAccessoryNone;
  if ([self tableView:tableView canSelectPointAtIndex:indexPath.row]) {
    CWPoint* otherPoint = _pointType == CWPointSelectionTypeFrom ? sharedModel.from : sharedModel.to;
    if ([point isEqual:otherPoint]) {
      cell.accessoryType = UITableViewCellAccessoryCheckmark;
      cell.textLabel.textColor = [UIColor editableColor];
    } else {
      cell.textLabel.textColor = [UIColor darkTextColor];
    }
  } else {
    cell.textLabel.textColor = [UIColor grayColor];
  }
  
  cell.textLabel.text = point.name;
  cell.imageView.image = [point imageForType];
	
  return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath;
{
  if (self.tableView == tableView) {
		CWPoint* point = [sharedModel.knownPoints objectAtIndex:indexPath.row];
    [pointSelectionDelegate pointSelectionController:self didSelectPoint:point];
  } else {
    CWPoint* point = [self.searchResultPoints objectAtIndex:indexPath.row];
		NSInteger index = [sharedModel.knownPoints indexOfObject:point];
    if (index != NSNotFound) {
    	point = [sharedModel.knownPoints objectAtIndex:index];
    } else {
      [sharedModel.knownPoints insertObject:point atIndex:0];
    }
    [pointSelectionDelegate pointSelectionController:self didSelectPoint:point];
  }
}


- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath;
{
  return self.tableView == tableView;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath;
{
	[sharedModel.knownPoints removeObjectAtIndex:indexPath.row];
  [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:indexPath.row inSection:0]] withRowAnimation:UITableViewRowAnimationFade];
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath;
{
  return self.tableView == tableView;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath;
{
	[sharedModel.knownPoints exchangeObjectAtIndex:fromIndexPath.row withObjectAtIndex:toIndexPath.row];
}


@end

